@@ -7,7 +7,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -24,11 +24,12 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef AGENT2D_SAMPLE_COACH_H
-#define AGENT2D_SAMPLE_COACH_H
+#ifndef SAMPLE_COACH_H
+#define SAMPLE_COACH_H
 
 #include <vector>
 #include <map>
+#include <set>
 
 #include <rcsc/types.h>
 
@@ -43,16 +44,14 @@ class SampleCoach
 private:
     typedef std::vector< const rcsc::PlayerType * > PlayerTypePtrCont;
 
-    //! used player type id set
+    int M_substitute_count; // substitution count after kick-off
     std::vector< int > M_available_player_type_id;
+    std::map< int, int > M_assigned_player_type_id;
 
-    //! player type assignment mapping. key: unum, value: type id
-    std::map< int, int > M_player_type_id;
-
-    //! opponent player type info sent as the known player type
     int M_opponent_player_types[11];
 
-    //! team graphic holder
+    double M_teammate_recovery[11];
+
     rcsc::TeamGraphic M_team_graphic;
 
 public:
@@ -64,6 +63,8 @@ public:
 
 private:
 
+    void updateRecoveryInfo();
+
     /*!
       \brief substitute player.
 
@@ -72,29 +73,22 @@ private:
     void doSubstitute();
 
     void doFirstSubstitute();
+    void doSubstituteTiredPlayers();
 
     void substituteTo( const int unum,
                        const int type );
 
-    void substituteTo( const std::vector< std::pair< int, int > > & types );
-
     int getFastestType( PlayerTypePtrCont & candidates );
 
-    /*!
-      \brief broadcast opponent players' player types
-     */
     void sayPlayerTypes();
 
-    /*!
-      \brief send team graphic tiles to rcssserver
-     */
     void sendTeamGraphic();
 
 protected:
 
     /*!
       You can override this method.
-      But you must call CoachrAgent::doInit() in this method.
+      But you must call PlayerAgent::doInit() in this method.
     */
     virtual
     bool initImpl( rcsc::CmdLineParser & cmd_parser );
